@@ -15,7 +15,7 @@ sub new {
         $self->do_system('git', 'submodule', 'update', '--init');
     }
     my @extra_compiler_flags = (qw/
-        -Wall -Wextra -Wno-duplicate-decl-specifier -Wno-parentheses
+        -Wall -Wextra -Wno-parentheses
         -Wno-unused -Wno-unused-parameter
     /, ('-I.', "-I$LIBZSTD_DIR"));
     my $ld = $self->config('ld');
@@ -39,7 +39,9 @@ sub ACTION_build {
     $self->ACTION_ppport_h() unless -e 'ppport.h';
     unless (-f "$LIBZSTD_DIR/libzstd.a") {
         local $ENV{CFLAGS} = '-O3 -fPIC';
-        $self->do_system('make' => '-C', $LIBZSTD_DIR, 'libzstd.a');
+        my $make = 'make';
+        $make = 'gmake' if $^O =~ /bsd$/ && $^O !~ /gnukfreebsd$/;
+        $self->do_system($make => '-C', $LIBZSTD_DIR, 'libzstd.a');
     }
     $self->SUPER::ACTION_build();
 }
